@@ -1,5 +1,3 @@
-/* -*- mode: c; kept-new-versions: 25; kept-old-versions: 20 -*- */
-
 /*   Clustering LARge Applications
      ~		~~~   ~
      Clustering program based upon the k-medoid approach,
@@ -7,8 +5,7 @@
      (for smaller data sets, please use program pam.)
  */
 
-/* $Id$
- * original Id: clara.f,v 1.10 2002/08/27 15:43:58 maechler translated by
+/* original Id: clara.f,v 1.10 2002/08/27 15:43:58 maechler translated by
  * f2c (version 20010821) and run through f2c-clean,v 1.10 2002/03/28
  */
 
@@ -16,6 +13,7 @@
 
 #include <R_ext/Print.h>/* for diagnostics */
 #include <R_ext/Random.h>/* when R's RNG is used */
+#include <R_ext/Utils.h>/* for interrupting */
 
 #include "cluster.h"
 #include "ind_2.h"
@@ -34,8 +32,8 @@ void clara(int *n,  /* = number of objects */
 	   double *valmd,/*[j]= missing value code (instead of NA) for x[,j]*/
 	   int *jtmd,	/* [j]= {-1,1};	 -1: x[,j] has NA; 1: no NAs in x[,j] */
 	   int *diss_kind,/*= {1,2};  1 : euclidean;  2 : manhattan*/
-	   int *rng_R,	/*  = {0,1};  0 : use clara's internal weak RNG;
-			 *	      1 : use R's RNG (and seed) */
+	   int/*logical*/ *rng_R,/*= {0,1};  0 : use clara's internal weak RNG;
+				  *	     1 : use R's RNG (and seed) */
 	   int *nrepr,
 	   int *nsel,
 	   int *nbest,/* x[nbest[j],] : the j-th obs in the final sample */
@@ -81,7 +79,8 @@ void clara(int *n,  /* = number of objects */
     else
 	n_sam = *nsam;
 
-    if(*trace_lev) Rprintf("C clara(): (nsam,n) = (%d,%d);%s\n", *nsam, *n,
+    if(*trace_lev) Rprintf("C clara(): (nsam,nran,n) = (%d,%d,%d);%s\n",
+			   *nsam, *nran, *n,
 			   full_sample ? " 'full_sample',":
 			   (lrg_sam ? " 'large_sample',": ""));
     if(*rng_R && !full_sample)
@@ -212,10 +211,10 @@ void clara(int *n,  /* = number of objects */
 		}
 		for (j = 0; j < *nsam; ++j)
 		    nsel[j] = nrepr[j];
-		if(*trace_lev >= 3) {
-		    Rprintf(".. nsel[1:%d]= ", *nsam);
-		    for (jk = 0; jk < *nsam; jk++) Rprintf("%d ",nsel[jk]);
-		}
+	    }
+	    if(*trace_lev >= 3) {
+		Rprintf(".. nsel[1:%d]= ", *nsam);
+		for (jk = 0; jk < *nsam; jk++) Rprintf("%d ",nsel[jk]);
 	    }
 	    if(*trace_lev) Rprintf(" -> dysta2()\n");
 	}
@@ -249,6 +248,8 @@ void clara(int *n,  /* = number of objects */
 	       /* dysma */tmp1, /*dysmb*/tmp2,
 	       /* beter[], only used here */&tmp[nsamb]);
 
+	if(*trace_lev >= 2) Rprintf(" bs2");
+
 	selec(*kk, *n, *jpp, *diss_kind, &zb, *nsam, has_NA, jtmd, valmd,
 	      nrepr, nsel, dys, x, nr, &nafs, ttd, radus, ratt,
 	      ntmp1, ntmp2, ntmp3, ntmp4, ntmp5, ntmp6, tmp1, tmp2);
@@ -261,8 +262,7 @@ void clara(int *n,  /* = number of objects */
 	}
 	else if(!kall || zba > zb) { /* 1st proper sample  or  new best */
 	    kall = TRUE;
-	    if(*trace_lev >= 2)
-		Rprintf(" 1st proper or new best: zb= %g", zb);
+	    if(*trace_lev >= 2) Rprintf(" 1st proper or new best:");
 	    zba = zb;
 	    for (jk = 0; jk < *kk; ++jk) {
 		ttbes[jk] = ttd	 [jk];
@@ -273,8 +273,8 @@ void clara(int *n,  /* = number of objects */
 	    for (js = 0; js < *nsam; ++js)
 		nbest[js] = nsel[js];
 	    sx = s;
-	} else if(*trace_lev >= 2) Rprintf(" zb= %g", zb);
-	if(*trace_lev >= 2) Rprintf("\n");
+	}
+	if(*trace_lev >= 2) Rprintf(" obj= %g\n", zb/rnn);
 
 	if(full_sample) break; /* out of resampling */
     }
@@ -341,7 +341,7 @@ void dysta2(int nsam, int jpp, int *nsel,
     double clk, d1;
 
     nlk = 0;
-    dys[0] = 0.;/* very first index; *is* used in ?? (it is, not in R!) */
+    dys[0] = 0.;/* very first index; *is* used because ind_2(i,i) |-> 0 ! */
     for (l = 1; l < nsam; ++l) {
 	lsel = nsel[l];
 	if(lsel <= 0 || lsel > n)
@@ -398,10 +398,8 @@ void bswap2(int kk, int nsam, int *nrepr,
 	    double *dys, double *sky, double s,
 	    double *dysma, double *dysmb, double *beter)
 {
-    int j, ja, k, kbest = -1, nbest = -1, nmax = -1;/* init for -Wall */
-    int nkj, njn, njaj, nny;
-
-    double ammax, small, asky, dzsky, dz, cmd;
+    int j, i, k, ij, kj, kbest = -1, nbest = -1;/* init for -Wall */
+    double small, asky, dzsky, dz;
 
     /* Parameter adjustments */
     --nrepr;
@@ -413,34 +411,37 @@ void bswap2(int kk, int nsam, int *nrepr,
 
 /* ====== first algorithm: BUILD. ====== */
 
-    for (j = 1; j <= nsam; ++j) {
-	nrepr[j] = 0;
-	dysma[j] = s;
+    for (i = 1; i <= nsam; ++i) {
+	nrepr[i] = 0;
+	dysma[i] = s;
     }
 
-    for(nny = 0; nny < kk; nny++) {
-	for (ja = 1; ja <= nsam; ++ja) {
-	    if (nrepr[ja] == 0) {
-		beter[ja] = 0.;
+    for(k = 0; k < kk; k++) {
+	int nmax = -1; /* -Wall */
+	double ammax, cmd;
+	ammax = 0.;
+	for (i = 1; i <= nsam; ++i) {
+	    if (nrepr[i] == 0) {
+		beter[i] = 0.;
 		for (j = 1; j <= nsam; ++j) {
-		    cmd = dysma[j] - dys[ ind_2(ja, j)];
+		    cmd = dysma[j] - dys[ ind_2(i, j)];
 		    if (cmd > 0.)
-			beter[ja] += cmd;
+			beter[i] += cmd;
+		}
+		if (ammax <= beter[i]) {
+		    /*	does < (instead of <= ) work too? -- NO! */
+		    ammax = beter[i];
+		    nmax = i;
 		}
 	    }
 	}
-	ammax = 0.;
-	for (ja = 1; ja <= nsam; ++ja) {
-	    if (nrepr[ja] == 0 && ammax <= beter[ja]) {
-		ammax = beter[ja];
-		nmax = ja;
-	    }
-	}
 	nrepr[nmax] = 1;
+
+	/* update dysma[] : dysma[j] = D(j, nearest_representative) */
 	for (j = 1; j <= nsam; ++j) {
-	    njn = ind_2(nmax, j);
-	    if (dysma[j] > dys[njn])
-		dysma[j] = dys[njn];
+	    ij = ind_2(nmax, j);
+	    if (dysma[j] > dys[ij])
+		dysma[j] = dys[ij];
 	}
     }
 
@@ -461,52 +462,51 @@ L60:
     for (j = 1; j <= nsam; ++j) {
 	dysma[j] = s;
 	dysmb[j] = s;
-	for (ja = 1; ja <= nsam; ++ja) {
-	    if (nrepr[ja] != 0) {
-		njaj = ind_2(ja, j);
-		if (dys[njaj] < dysma[j]) {
+	for (i = 1; i <= nsam; ++i) {
+	    if (nrepr[i] != 0) {
+		ij = ind_2(i, j);
+		if (dysma[j] > dys[ij]) {
 		    dysmb[j] = dysma[j];
-		    dysma[j] = dys[njaj];
-		} else if (dys[njaj] < dysmb[j])
-		    dysmb[j] = dys[njaj];
+		    dysma[j] = dys[ij];
+		} else if (dysmb[j] > dys[ij]) {
+		    dysmb[j] = dys[ij];
+		}
 	    }
 	}
     }
 
     dzsky = 1.;
-    for (k = 1; k <= nsam; ++k) {
-	if (nrepr[k] != 1) {
-	    for (ja = 1; ja <= nsam; ++ja) {
-		if (nrepr[ja] != 0) {
-		    dz = 0.;
-		    for (j = 1; j <= nsam; ++j) {
-			njaj = ind_2(ja, j);
-			nkj  = ind_2(k, j);
-			if (dys[njaj] == dysma[j]) {
-			    small = dysmb[j];
-			    if (small > dys[njaj])
-				small = dys[nkj];
-			    dz = dz - dysma[j] + small;
-			}
-			else if (dys[nkj] < dysma[j])
-			    dz = dz - dysma[j] + dys[nkj];
-		    }
-		    if (dz < dzsky) {
-			dzsky = dz;
-			kbest = k;
-			nbest = ja;
-		    }
+    for (k = 1; k <= nsam; ++k) if (nrepr[k] == 0) {
+	for (i = 1; i <= nsam; ++i) if (nrepr[i] != 0) {
+	    dz = 0.;
+	    /* dz := T_{ih} := sum_j C_{jih}  [p.104] : */
+	    for (j = 1; j <= nsam; ++j) {
+		ij = ind_2(i, j);
+		kj  = ind_2(k, j);
+		if (dys[ij] == dysma[j]) {
+		    small = dysmb[j] > dys[ij] ? dys[kj] : dysmb[j];
+		    dz += (- dysma[j] + small);
 		}
+		else if (dys[kj] < dysma[j])
+		    dz += (- dysma[j] + dys[kj]);
+	    }
+	    if (dzsky > dz) {
+		dzsky = dz;
+		kbest = k;
+		nbest = i;
 	    }
 	}
     }
-    if (dzsky >= 0.)
-	return;
 
-    nrepr[kbest] = 1;
-    nrepr[nbest] = 0;
-    *sky += dzsky;
-    goto L60;
+    /* once had some 64-bit compiler / data configuration that looped forever*/
+    R_CheckUserInterrupt();
+
+    if (dzsky < 0.) { /* found an improving swap */
+	nrepr[kbest] = 1;
+	nrepr[nbest] = 0;
+	*sky += dzsky;
+	goto L60;
+    }
 
 } /* End of bswap2() -------------------------------------------------- */
 
